import {CHARSET, CHARS} from '@mathjax/font-tools/js/CharMap.js';
import {Font} from '@mathjax/font-tools/js/Font.js';
import {Variants} from '@mathjax/font-tools/js/Variant.js';
import {Delimiters} from '@mathjax/font-tools/js/Delimiters.js';
import {FontDef} from '@mathjax/font-tools/js/CommonFont.js';
import {SVGFont} from '@mathjax/font-tools/js/SVGFont.js';
import {CHTMLFont} from '@mathjax/font-tools/js/CHTMLFont.js';
import {Components} from '@mathjax/font-tools/js/Components.js';
import {FontData} from 'mathjax-full/mjs/output/common/FontData.js';

import {GenericFont as GenericSVGFont} from '@mathjax/fonts/generic/mjs/svg.js';
import {GenericFont as GenericCHTMLFont} from '@mathjax/fonts/generic/mjs/chtml.js';

/***********************************************************************************/

try {

  Font.load({
    'MD-ss': ['fonts/dsss10.otf', {}],
    'MD-rm': ['fonts/dsrom10.otf', {}]
  });

  /***********************************************************************************/

  const MathJaxDsfontVariants = Variants.define({
    '-ds-ss': ['MD-ss', [
      CHARS.MapFrom(0x41, CHARSET.MathDoubleStruckUC),
      CHARS.Map({0x1D7D9: 0x31, 0x1D552: 0x61, 0x1D559: 0x68, 0x1D55C: 0x6B})
     ]],
    '-ds-rm': ['MD-rm', [
      CHARS.MapFrom(0x41, CHARSET.MathDoubleStruckUC),
      CHARS.Map({0x1D7D9: 0x31, 0x1D552: 0x61, 0x1D559: 0x68, 0x1D55C: 0x6B})
     ]]
  });

  /***********************************************************************************/

  const MathJaxDsfontDelimiters = {
    font: 'MD-rm',
    variants: MathJaxDsfontVariants,
    sizeVariants: ['normal'],
    stretchVariants: ['normal'],
    readMathTable: false,
  };

  /***********************************************************************************/

  const MathJaxDsfontData: FontDef = {
    name: 'MathJaxDsfont',
    prefix: 'MD',
    variants: MathJaxDsfontVariants,
    delimiters: null,
    variantFallbacks: {
      '-ds-rm': ['double-struck'],
      '-ds-ss': ['bold-sans-serif']
    },
    variantSmp: {
      '-ds-ss': 'double-struck'
    }
  };

  const MathJaxDsfontSVGData: FontDef = {
    ...MathJaxDsfontData,
    delimiters: Delimiters.define({
      ...MathJaxDsfontDelimiters,
      extends: {
        fontClass: GenericSVGFont as typeof FontData,
        path: '@mathjax/fonts/generic/mjs/svg.js'
      },
    })
  };

  const MathJaxDsfontCHTMLData: FontDef = {
    ...MathJaxDsfontData,
    delimiters: Delimiters.define({
      ...MathJaxDsfontDelimiters,
      extends: {
        fontClass: GenericCHTMLFont as typeof FontData,
        path: '@mathjax/fonts/generic/mjs/chtml.js'
      },
    })
  };

  SVGFont.define(MathJaxDsfontSVGData).writeExtension('mathjax-dsfont');
  Components.define('svg', MathJaxDsfontSVGData).writeExtension();

  CHTMLFont.define(MathJaxDsfontCHTMLData).writeExtension('mathjax-dsfont').makeWoffFonts('MD-rm', false);
  Components.define('chtml', MathJaxDsfontCHTMLData).writeExtension();

} catch (err) {
  console.log(err);
  process.exit(1);
}
